@@ -55,30 +55,6 @@
 
 static Lum::Def::AppInfo info;
 
-static std::wstring DoubleToWString(double value, double unit)
-{
-  std::stringstream buffer;
-
-  buffer.imbue(std::locale(""));
-  buffer.setf(std::ios::fixed);
-  buffer << std::setprecision(1);
-
-  if (ceil(value*unit)>=1024*1024*1024*0.5) {
-    buffer << (value*unit)/(1024*1024*1024) << " TB";
-  }
-  else if (ceil(value*unit)>=1024*1024*0.5) {
-    buffer << (value*unit)/(1024*1024) << " GB";
-  }
-  else if (ceil(value*unit)>=1024*0.5) {
-    buffer << (value*unit)/1024 << " MB";
-  }
-  else {
-    buffer << value*unit << " KB";
-  }
-
-  return Lum::Base::StringToWString(buffer.str());
-}
-
 static Lum::TextValue* GetTextValueLeft(const Lum::Model::StringRef& value)
 {
   Lum::TextValue   *text;
@@ -130,16 +106,19 @@ public:
   };
 };
 
-class Usage : public Lum::Control
+typedef Lum::Model::StdTable<DiskInfo>      DiskInfoModel;
+typedef Lum::Base::Reference<DiskInfoModel> DiskInfoModelRef;
+
+class DiskInfoObject : public Lum::Object
 {
 private:
-  Lum::Model::DoubleRef model;
-  Lum::OS::FontRef      font;
-  Lum::OS::ColorRef     red;
-  Lum::OS::ColorRef     green;
+  DiskInfo          diskInfo;
+  Lum::OS::FontRef  font;
+  Lum::OS::ColorRef red;
+  Lum::OS::ColorRef green;
 
 public:
-  Usage()
+  DiskInfoObject()
   : font(Lum::OS::display->GetFont()),
     red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
     green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
@@ -147,30 +126,28 @@ public:
     // no code
   }
 
-  bool SetModel(Lum::Base::Model* model)
+  void SetDiskInfo(const DiskInfo& diskInfo)
   {
-    this->model=dynamic_cast<Lum::Model::Double*>(model);
-
-    Control::SetModel(this->model);
-
-    return this->model.Valid();
+    this->diskInfo=diskInfo;
   }
 
   void CalcSize()
   {
-    minWidth=2*Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
-             3*font->StringWidth(L"m",Lum::OS::Font::normal);
-    minHeight=font->pixelHeight+2*Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
+    minWidth=10*font->StringWidth(L"m",Lum::OS::Font::normal);
+    minHeight=Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+              font->height+
+              font->height+
+              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
 
     width=minWidth;
     height=minHeight;
 
-    Control::CalcSize();
+    Object::CalcSize();
   }
 
   void Draw(int x, int y, size_t w, size_t h)
   {
-    Control::Draw(x,y,w,h);
+    Object::Draw(x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
       return;
@@ -180,11 +157,24 @@ public:
 
     Lum::OS::DrawInfo *draw=GetDrawInfo();
 
-    if (!model.Valid() || model->IsNull()) {
-      return;
-    }
+    std::wstring sizeText=Lum::Base::ByteSizeToWString(diskInfo.avail*diskInfo.unit*1024)+L" free";
+    size_t       sizeTextWidth=font->StringWidth(sizeText);
 
-    double percent=model->Get();
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+    draw->DrawString(this->x,
+                     this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+font->ascent,
+                     diskInfo.dir);
+
+    draw->DrawString(this->x+width-sizeTextWidth+1,
+                     this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+font->ascent,
+                     sizeText);
+
+    draw->PopForeground();
+    draw->PopFont();
+
+
+    double percent=(diskInfo.blocks-diskInfo.avail)/diskInfo.blocks;
 
     if (percent>=0.9) {
       draw->PushForeground(red);
@@ -193,61 +183,41 @@ public:
       draw->PushForeground(green);
     }
 
-    draw->FillRectangle(this->x,this->y+1,
-                        (size_t)(width*percent),height-2);
+    draw->FillRectangle(this->x,
+                        this->y+
+                        Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                        font->height,
+                        (size_t)(width*percent),
+                        font->height);
 
     draw->PopForeground();
   }
-
-  void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
-  {
-    if (model==this->model) {
-      Redraw();
-    }
-    Control::Resync(model,msg);
-  }
 };
 
-typedef Lum::Model::StdTable<DiskInfo>      DiskInfoModel;
-typedef Lum::Base::Reference<DiskInfoModel> DiskInfoModelRef;
-
 class DiskInfoDataProvider : public DiskInfoModel::DataProvider
 {
 private:
-  Usage                 *percent;
-  Lum::Model::DoubleRef data;
+  DiskInfoObject *object;
 
 public:
   DiskInfoDataProvider()
-  : percent(new Usage()),
-    data(new Lum::Model::Double())
+  : object(new DiskInfoObject())
   {
-    percent->SetFlex(true,true);
-    percent->SetMinWidth(Lum::Base::Size::stdCharWidth,15);
-    percent->SetModel(data);
+    object->SetFlex(true,true);
+    object->SetMinWidth(Lum::Base::Size::stdCharWidth,15);
   }
 
   virtual ~DiskInfoDataProvider()
   {
-    delete percent;
-  }
-
-  std::wstring GetString(const DiskInfoModel::Iterator& iter, size_t column) const
-  {
-    switch (column) {
-    case 1:
-      return iter->dir;
-    default:
-      return L"";
-    }
+    delete object;
   }
 
   Lum::Object* GetObject(const DiskInfoModel::Iterator& iter, size_t column) const
   {
     switch (column) {
-    case 2:
-      data->Set((iter->blocks-iter->avail)/iter->blocks);
-      return percent;
+    case 1:
+      object->SetDiskInfo(*iter);
+      return object;
     default:
       return NULL;
     }
@@ -319,6 +289,8 @@ public:
     Lum::PercentPie       *percent;
     Lum::Model::HeaderRef headerModel;
 
+    tableModel->SetEmptyText(L"(No partitions found)");
+
     panel=Lum::VPanel::Create(true,true);
     panel->SetWidth(Lum::Base::Size::workHRel,20);
     panel->SetHeight(Lum::Base::Size::workVRel,40);
@@ -340,8 +312,7 @@ public:
     panel->AddSpace();
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Dir",Lum::Base::Size::stdCharWidth,25);
-    headerModel->AddColumn(L"Size",Lum::Base::Size::stdCharWidth,15,true);
+    headerModel->AddColumn(L"Dir",Lum::Base::Size::stdCharWidth,25,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
@@ -349,7 +320,7 @@ public:
     table->SetHeaderModel(headerModel);
     table->SetSelection(selection);
     table->SetDoubleClickAction(showMap);
-    table->SetShowHeader(true);
+    table->SetShowHeader(false);
     table->GetTableView()->SetAutoFitColumns(true);
     panel->Add(table);
 
@@ -440,9 +411,10 @@ public:
       device->Set(iter->name);
       dir->Set(iter->dir);
 
-      total->Set(DoubleToWString(iter->blocks,iter->unit));
-      used->Set(DoubleToWString(iter->blocks-iter->free,iter->unit));
-      free->Set(DoubleToWString(iter->avail,iter->unit)+L" (root: "+DoubleToWString(iter->free,iter->unit)+L")");
+      total->Set(Lum::Base::ByteSizeToWString(iter->blocks*iter->unit*1024));
+      used->Set(Lum::Base::ByteSizeToWString((iter->blocks-iter->free)*iter->unit*1024));
+      free->Set(Lum::Base::ByteSizeToWString(iter->avail*iter->unit*1024)+
+                L" (root: "+Lum::Base::ByteSizeToWString(iter->free*iter->unit*1024)+L")");
 
       data->Enable();
       data->Set(0,(iter->blocks-iter->free)/iter->blocks); // used
